# -*- coding: utf-8 -*-
u"""Расширение поведения интерфейсов."""

import objectpack


class BaseEditWinExtender(object):

    u"""
    Базовый объект для расширения окон редактирования.

    ``_extend_edit_win`` - добавление и размещение контролов
    ``bind_to_object`` - заполнение объекта данными из полей формы
    ``bind_from_object`` - заполнение полей формы из объекта
    """

    # основная модель, которая редактируется в окне (расширяемая модель)
    model = None
    # список имен полей доп.модели (которой расширяется основная модель model)
    model_fields = None
    # реестр моделей-паков (обычно observer)
    # не обязательно, если не будет использоваться ``model_fields_to_controls``
    model_register = None

    def __init__(self, win):
        # ожидается окно
        self._win = win
        self._extend_edit_win()

    def _extend_edit_win(self):
        u"""
        Расширение формы окна.

        ..code::

            for fld in self.model_fields_to_controls(self.model_fields):
                self._win.form.items.append(fld)

        """
        raise NotImplementedError()

    def model_fields_to_controls(self, fields):
        u"""Шорткат для генерации контролов."""
        assert self.model, u'No model defined in Extender!'
        return objectpack.ui.model_fields_to_controls(
            self.model,
            self._win,
            field_list=fields,
            model_register=self.model_register
        )

    @classmethod
    def bind_to_object(cls, instance, context):
        u"""
        Заполнение полей ``model_fields`` модели instance по полям формы.

        :param instance: инстанс модели, которая расширяет основную ``model``
        :param context: контекст
        :type context: m3.actions.context.DeclarativeActionContext
        """
        assert cls.model_fields, u'No model_fields defined in Extender'

        for field_name in cls.model_fields:
            try:
                value = getattr(context, field_name)
            except AttributeError:
                value = None

            setattr(instance, field_name, value)

    def bind_from_object(self, instance):
        u"""
        Заполнение полей ``model_fields`` формы по полям модели instance.

        :param instance: инстанс модели, которая расширяет основную ``model``
        """
        assert self.model_fields, u'No model_fields defined in Extender'

        for name in self.model_fields:
            field = self._win.find_by_name(name)
            value = getattr(instance, field.name)
            if value:
                field.value = value
